@@ -44,7 +44,7 @@ PYTHON3 = sys.version_info[0] > 2
 
 DEBUG = False
 
-DEFAULT_CHARSET = 'latin1'
+DEFAULT_CHARSET = 'utf8'
 
 def byte2int(b):
     if isinstance(b, int):
@@ -177,11 +177,10 @@ class Connection(object):
 
         if not issubclass(errorclass, Error):
             raise Error(errorclass, errorvalue)
+        elif isinstance(errorvalue, errorclass):
+            raise errorvalue
         else:
-            if PYTHON3:
-                raise errorclass(errorvalue)
-            else:
-                raise errorclass(errorvalue[0], errorvalue[1])
+            raise errorclass(*errorvalue)
 
 
     def __init__(self, host="localhost", user=None, passwd="",
@@ -478,7 +477,7 @@ class Connection(object):
         #send_data = struct.pack('<i', len(sql) + 1) + command + sql
         # could probably be more efficient, at least it's correct
         if not self.socket:
-            self.errorhandler(None, InterfaceError, "(0, '')")
+            self.errorhandler(None, InterfaceError, (-1, 'socket not found'))
 
         if ((PYTHON3 and isinstance(sql, str)) or 
             (not PYTHON3 and  isinstance(sql, unicode))):
@@ -503,7 +502,7 @@ class Connection(object):
             raise ValueError("Did not specify a username")
 
         charset_id = charset_by_name(self.charset).id
-        self.user = self.user.encode(self.charset)
+        user = self.user.encode(self.charset)
 
         data_init = struct.pack('<i', self.client_flag) + struct.pack("<I", 1) + \
                      int2byte(charset_id) + int2byte(0)*23
@@ -523,7 +522,7 @@ class Connection(object):
                                                  cert_reqs=ssl.CERT_REQUIRED,
                                                  ca_certs=self.ca)
 
-        data = data_init + self.user+int2byte(0) + _scramble(self.password.encode(self.charset), self.salt)
+        data = data_init + user+int2byte(0) + _scramble(self.password.encode(self.charset), self.salt)
 
         if self.db:
             self.db = self.db.encode(self.charset)
