@@ -2,20 +2,16 @@ import re
 import datetime
 import time
 import sys
+from decimal import Decimal
 
 from cymysql.constants import FIELD_TYPE, FLAG
-from cymysql.charset import charset_by_id
+try:
+    from cymysql.charsetx import charset_by_id
+except ImportError:
+    from cymysql.charset import charset_by_id
 
 PYTHON3 = sys.version_info[0] > 2
 
-try:
-    set
-except NameError:
-    try:
-        from sets import BaseSet as set
-    except ImportError:
-        from sets import Set as set
-
 ESCAPE_REGEX = re.compile(r"[\0\n\r\032\'\"\\]")
 ESCAPE_MAP = {'\0': '\\0', '\n': '\\n', '\r': '\\r', '\032': '\\Z',
               '\'': '\\\'', '"': '\\"', '\\': '\\\\'}
@@ -97,6 +93,9 @@ def escape_date(obj):
 def escape_struct_time(obj):
     return escape_datetime(datetime.datetime(*obj[:6]))
 
+def escape_decimal(obj):
+    return str(obj)
+
 def convert_datetime(connection, field, obj):
     """Returns a DATETIME or TIMESTAMP column value as a datetime object:
 
@@ -272,9 +271,12 @@ def convert_characters(connection, field, data):
     if field.flags & FLAG.SET:
         return convert_set(data.decode(field_charset))
     if field.flags & FLAG.BINARY:
-        return data
+        if PYTHON3 and field_charset != 'binary':
+            return data.decode(field_charset)
+        else:
+            return data
 
-    if connection.use_unicode:
+    if connection.use_unicode or PYTHON3:
         data = data.decode(field_charset)
     elif connection.charset != field_charset:
         data = data.decode(field_charset)
@@ -293,44 +295,15 @@ def convert_long(connection, field, data):
 def convert_float(connection, field, data):
     return float(data)
 
-try:
-    from convertersx import (
-        escape_item,
-        escape_dict,
-        escape_sequence,
-        escape_set,
-        escape_bool,
-        escape_object,
-        escape_int,
-        escape_long,
-        escape_float,
-        escape_string,
-        escape_unicode,
-        escape_None,
-        escape_timedelta,
-        escape_time,
-        escape_datetime,
-        escape_date,
-        escape_struct_time,
-        convert_datetime,
-        convert_timedelta,
-        convert_time,
-        convert_date,
-        convert_mysql_timestamp,
-        convert_set,
-        convert_bit,
-        convert_characters,
-        convert_int,
-        convert_long,
-        convert_float,
-    )
-except:
-    pass
+def convert_decimal(connection, field, data):
+    data = data.decode(connection.charset)
+    return Decimal(data)
 
 encoders = {
         bool: escape_bool,
         int: escape_int,
         float: escape_float,
+        Decimal: escape_decimal,
         str: escape_string,
         tuple: escape_sequence,
         list:escape_sequence,
@@ -343,7 +316,6 @@ encoders = {
         datetime.time : escape_time,
         time.struct_time : escape_struct_time,
         }
-
 if not PYTHON3:
     encoders[unicode] = escape_unicode
     encoders[long] = escape_long
@@ -359,6 +331,8 @@ decoders = {
         FIELD_TYPE.NEWDECIMAL: convert_float,
         FIELD_TYPE.LONGLONG: convert_long,
         FIELD_TYPE.INT24: convert_int,
+        FIELD_TYPE.DECIMAL:  convert_decimal,
+        FIELD_TYPE.NEWDECIMAL: convert_decimal,
         FIELD_TYPE.YEAR: convert_int,
         FIELD_TYPE.TIMESTAMP: convert_mysql_timestamp,
         FIELD_TYPE.DATETIME: convert_datetime,
@@ -379,18 +353,3 @@ decoders = {
         }
 conversions = decoders  # for MySQLdb compatibility
 
-try:
-    # python version > 2.3
-    from decimal import Decimal
-    def convert_decimal(connection, field, data):
-        data = data.decode(connection.charset)
-        return Decimal(data)
-    decoders[FIELD_TYPE.DECIMAL] = convert_decimal
-    decoders[FIELD_TYPE.NEWDECIMAL] = convert_decimal
-
-    def escape_decimal(obj):
-        return unicode(obj)
-    encoders[Decimal] = escape_decimal
-
-except ImportError:
-    pass
