@@ -54,7 +54,6 @@ class MysqlPacket(object):
 
     def __init__(self, connection):
         self.connection = connection
-        self.sock_fd = connection.sock_fd
         self.__position = 0
         self.__recv_packet()
 
@@ -84,12 +83,23 @@ class MysqlPacket(object):
             raise OperationalError(2013, "Lost connection to MySQL server during query")
 
         self.__data = recv_data
+        self.__data_length = bytes_to_read
   
     def get_all_data(self): return self.__data
   
     def read(self, size):
+        return self._read(size)
+
+    def _read(self, size):
         """Read the first 'size' bytes in packet and advance cursor past them."""
-        result = self.peek(size)
+        if self.__position + size > self.__data_length:
+            error = ('Result length not requested length:\n'
+                 'Expected=%s.  Actual=%s.  Position: %s.  Data Length: %s'
+                 % (size, self.__data_length - self.__position,
+                    self.__position, self.__data_length))
+            raise AssertionError(error)
+        result = self.__data[self.__position:(self.__position+size)]
+
         self.advance(size)
         return result
   
@@ -105,28 +115,18 @@ class MysqlPacket(object):
     def advance(self, length):
         """Advance the cursor in data buffer 'length' bytes."""
         new_position = self.__position + length
-        if new_position < 0 or new_position > len(self.__data):
+        if new_position < 0 or new_position > self.__data_length:
             raise Exception('Invalid advance amount (%s) for cursor.  '
                         'Position=%s' % (length, new_position))
         self.__position = new_position
   
     def rewind(self, position=0):
         """Set the position of the data buffer cursor to 'position'."""
-        if position < 0 or position > len(self.__data):
+        if position < 0 or position > self.__data_length:
             raise Exception(
                     "Invalid position to rewind cursor to: %s." % position)
         self.__position = position
   
-    def peek(self, size):
-        """Look at the first 'size' bytes in packet without moving cursor."""
-        result = self.__data[self.__position:(self.__position+size)]
-        if len(result) != size:
-            error = ('Result length not requested length:\n'
-                 'Expected=%s.  Actual=%s.  Position: %s.  Data Length: %s'
-                 % (size, len(result), self.__position, len(self.__data)))
-            raise AssertionError(error)
-        return result
-  
     def get_bytes(self, position, length=1):
         """Get 'length' bytes starting at 'position'.
   
@@ -144,20 +144,23 @@ class MysqlPacket(object):
         Length coded numbers can be anywhere from 1 to 9 bytes depending
         on the value of the first byte.
         """
-        c = ord(self.read(1))
+        c = ord(self._read(1))
         if c == NULL_COLUMN:
-            return None
+            return -1
         if c < UNSIGNED_CHAR_COLUMN:
             return c
         elif c == UNSIGNED_SHORT_COLUMN:
-            return unpack_uint16(self.read(UNSIGNED_SHORT_LENGTH))
+            return unpack_uint16(self._read(UNSIGNED_SHORT_LENGTH))
         elif c == UNSIGNED_INT24_COLUMN:
-            return unpack_uint24(self.read(UNSIGNED_INT24_LENGTH))
+            return unpack_uint24(self._read(UNSIGNED_INT24_LENGTH))
         elif c == UNSIGNED_INT64_COLUMN:
             # TODO: what was 'longlong'?  confirm it wasn't used?
-            pass
+            return -1
   
     def read_length_coded_string(self):
+        return self._read_length_coded_string()
+
+    def _read_length_coded_string(self):
         """Read a 'Length Coded String' from the data buffer.
 
         A 'Length Coded String' consists first of a length coded
@@ -165,10 +168,18 @@ class MysqlPacket(object):
         that many bytes of binary data.  (For example "cat" would be "3cat".)
         """
         length = self.read_length_coded_binary()
-        if length is None:
+        if length < 0:
             return None
-        return self.read(length)
-  
+        return self._read(length)
+
+    def read_decode_data(self, decoders, field):
+        data = self._read_length_coded_string()
+        if data != None:
+            func = decoders.get(field.type_code)
+            if func:
+                return func(self.connection, field, data)
+        return None
+ 
     def is_ok_packet(self):
         return ord(self.get_bytes(0)) == 0
 
@@ -183,7 +194,7 @@ class MysqlPacket(object):
         if ord(self.get_bytes(0)) == 255:
             self.rewind()
             self.advance(1)  # field_count == error (we already know that)
-            errno = unpack_uint16(self.read(2))
+            errno = unpack_uint16(self._read(2))
             return errno, self.__data
         return 0, None
 
@@ -191,10 +202,13 @@ class MysqlPacket(object):
         self.advance(1)  # field_count (always '0')
         affected_rows = self.read_length_coded_binary()
         insert_id = self.read_length_coded_binary()
-        server_status = unpack_uint16(self.read(2))
-        warning_count = unpack_uint16(self.read(2))
+        server_status = unpack_uint16(self._read(2))
+        warning_count = unpack_uint16(self._read(2))
         message = self.read_all()
-        return (affected_rows, insert_id, server_status, warning_count, message)
+        return (None if affected_rows < 0 else affected_rows,
+                None if insert_id < 0 else insert_id,
+                server_status, warning_count, message)
+
 
 
 class FieldDescriptorPacket(MysqlPacket):
@@ -213,18 +227,18 @@ class FieldDescriptorPacket(MysqlPacket):
     
         This is compatible with MySQL 4.1+ (not compatible with MySQL 4.0).
         """
-        self.catalog = self.read_length_coded_string()
-        self.db = self.read_length_coded_string()
-        self.table_name = self.read_length_coded_string()
-        self.org_table = self.read_length_coded_string()
-        self.name = self.read_length_coded_string().decode(self.connection.charset)
-        self.org_name = self.read_length_coded_string()
+        self.catalog = self._read_length_coded_string()
+        self.db = self._read_length_coded_string()
+        self.table_name = self._read_length_coded_string()
+        self.org_table = self._read_length_coded_string()
+        self.name = self._read_length_coded_string().decode(self.connection.charset)
+        self.org_name = self._read_length_coded_string()
         self.advance(1)  # non-null filler
-        self.charsetnr = unpack_uint16(self.read(2))
-        self.length = unpack_uint32(self.read(4))
-        self.type_code = ord(self.read(1))
-        self.flags = unpack_uint16(self.read(2))
-        self.scale = ord(self.read(1))  # "decimals"
+        self.charsetnr = unpack_uint16(self._read(2))
+        self.length = unpack_uint32(self._read(4))
+        self.type_code = ord(self._read(1))
+        self.flags = unpack_uint16(self._read(2))
+        self.scale = ord(self._read(1))  # "decimals"
         self.advance(2)  # filler (always 0x00)
     
         # 'default' is a length coded binary and is still in the buffer?
@@ -296,27 +310,21 @@ class MySQLResult(object):
         self._get_descriptions()
         self._read_rowdata_packet()
 
-    def _field_data(self, packet, decoders, field):
-        data = packet.read_length_coded_string()
-        if data != None and field.type_code in decoders:
-            return decoders[field.type_code](self.connection, field, data)
-        return None
-
     # TODO: implement this as an iteratable so that it is more
     #       memory efficient and lower-latency to client...
     def _read_rowdata_packet(self):
       """Read a rowdata packet for each data row in the result set."""
-      decoders = self.connection.decoders
       rows = []
+      decoders = self.connection.decoders
       while True:
         packet = self.connection.read_packet()
         if packet.is_eof_packet():
-            self.warning_count = packet.read(2)
+            self.warning_count = unpack_uint16(packet.read(2))
             server_status = unpack_uint16(packet.read(2))
             self.has_next = (server_status
                              & SERVER_STATUS.SERVER_MORE_RESULTS_EXISTS)
             break
-        rows.append(tuple([self._field_data(packet, decoders, self.fields[i])
+        rows.append(tuple([packet.read_decode_data(decoders, self.fields[i])
                                             for i in range(len(self.fields))]))
 
       self.affected_rows = len(rows)
